/* -*- Mode: C; c-file-style: "gnu" -*-
  scandir.c -- if scandir() is missing, make a replacement
  Created: Petter Reinholdtsen <pere@td.org.uit.no>, 1998-03-24
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifndef HAVE_SCANDIR

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <dirent.h>

/*
 * XXX This is a simple hack version which doesn't sort the data, and
 * just passes all unsorted.
 */

int
alphasort(const void *a, const void *b)
{
  return 0;
}

int 
scandir(const char *dir, struct dirent ***namelist,
	int (*select)(const struct dirent *),
	int (*compar)(const void *, const void *))
{
  DIR *d = opendir(dir);
  struct dirent *current;
  struct dirent **names;
  int count = 0;
  int pos = 0;
  int result = -1;
  off_t start;

  if (NULL == d)
    return -1;

  start = telldir(d);

  while (NULL != readdir(d)) count++;

  names = malloc(sizeof(struct dirent *) * count);

  seekdir(d, start);

  while (NULL != (current = readdir(d))) {
    if ( NULL == select || select(current) ) {
      struct dirent *copyentry = malloc(sizeof(struct dirent));

      /* This is a hack for Solaris which lets d_name be char[1] */
      memcpy(copyentry, current, 
	     sizeof(struct dirent) - sizeof(current->d_name)
	     +strlen(current->d_name)+1);

      names[pos] = copyentry;
      pos++;
    }
  }
  result = closedir(d);

  if (pos != count)
    names = realloc(names, sizeof(struct dirent *)*pos);

  *namelist = names;

  return pos;
}

#endif /* HAVE_SCANDIR */
