/* -*- Mode: C; c-file-style: "gnu" -*-
   gc.h -- stuff used by our garbage collector.
   Created: Chris Toshok <toshok@hungry.com>, 10-Aug-1997
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */

#ifndef _gc_h
#define _gc_h

#include "jni.h"
#include "native-threads.h"

typedef struct GC_obj {
  /* used for the various lists of gc-able objects we keep around,
     like the finalizer_q, the local ref's per thread, and the global
     ref's */
  struct GC_obj *prev;
  struct GC_obj *next;

  int gc_flags;

  size_t size;

  /* called so that the gc object can free itself. */
  void (*finalize)(struct GC_obj*);
} GC_obj;

typedef struct GC_state {
  JavaVM *vm; 						/* backpointer to our vm. */

  HMonitor mon;						/* this GC_state's monitor */

  jint num_gcs;						/* number of collections since we were created. */
  jint num_finalizer_runs;			/* number of finalizer runs since we were created. */

  /* these are the totals that are amassed since the creation
     of the GC_state */
  jint total_num_bytes_allocated;
  jint total_num_bytes_collected;
  jint total_num_classes_allocated;
  jint total_num_classes_finalized;
  jint total_num_arrays_allocated;
  jint total_num_arrays_finalized;
  jint total_num_objects_allocated;
  jint total_num_objects_finalized;

  /* these totals are per collection.  Meaning that they get
     zero'ed after a collection takes place.  They are used 
     to determine when a new collection should happen. */
  jint interval_num_bytes_allocated;
  jint interval_num_classes_allocated;
  jint interval_num_arrays_allocated;
  jint interval_num_objects_allocated;
  
  int verbose_gc;
  int class_gc_enabled;

  /* these are used only for debugging spew during collection. */
  int num_classes_reachable;
  int num_arrays_reachable;
  int num_objects_reachable;
  
  GC_obj *finalizer_q;
  GC_obj *global_refs;

  /* those objects that never go away (until the GC state is deleted). */
  GC_obj *roots;
} GC_state;

#define GC_FLAG_OBJECT		0x1 /* it's an obj_thunk */
#define GC_FLAG_CLASS		0x2 /* it's a ClazzFile */
#define GC_FLAG_ARRAY		0x4 /* it's an array */
#define GC_FLAG_MARKED		0x8 /* reached during the current gc */

extern GC_state* GC_createGCState();
extern void GC_deleteGCState(GC_state* state);

extern void GC_runCollector(JavaVM *vm);
extern void GC_runFinalizers(JavaVM *vm);

extern void GC_finalize(JavaVM *vm, GC_obj *obj);
extern void GC_deleteGlobal(JavaVM *vm, GC_obj *obj);
extern void GC_deleteThreadLocal(JavaVM *vm, GC_obj *obj);

extern void GC_printStats(JavaVM *vm);

extern void GC_addReference(JavaVM *vm, GC_obj *obj);
extern void GC_addRoot(JavaVM *vm, GC_obj *obj);

#endif /* _gc_h */
