/* -*- Mode: C; c-file-style: "gnu" -*-
   objects.h -- Our low-level representation of objects.
   Created: Chris Toshok <toshok@hungry.com>, 10-Aug-1997
 */

/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#ifndef _objects_h
#define _objects_h

#include "gc.h"
#include "ClazzFile.h"
#include "op_stack.h"

typedef struct instance_data {
  JavaStackItem *fields;
  HMonitor monitor;
} instance_data;

typedef struct obj_thunk {
  GC_obj _gc_obj; /* needs to be here for this type to be gc'able */

  int is_deepest_child; /* true for the obj_thunk of the actual class of
			   this object. */
  instance_data *_instance;
  ClazzFile *_static;
} obj_thunk;

#define INSTANCE_DATA(c)	(((obj_thunk*)c)->_instance)
#define STATIC_DATA(c)		(((obj_thunk*)c)->_static)

#define SUPERCLASS_REF(p) (((obj_thunk*)(p)) ++)
#define SUBCLASS_REF(p) (((obj_thunk*)(p)) ++)

#define MAKE_POST(p) { ((obj_thunk*)(p))->_instance = NULL; \
    ((obj_thunk*)(p))->_static = NULL; }

#define IS_OBJECT_REF(p) ((((obj_thunk*)(p))->_instance != NULL) \
			  && (((obj_thunk*)(p))->_static != NULL))
#define IS_POST(p) ((((obj_thunk*)(p))->_instance == NULL) \
		    && (((obj_thunk*)(p))->_static == NULL))

     extern obj_thunk* new_classobject(JNIEnv *env, ClazzFile *cf);
     extern obj_thunk* new_object(JNIEnv *env, ClazzFile *cf);
     extern obj_thunk* clone_object(JNIEnv *env, obj_thunk *obj);

     extern void object_finalize(GC_obj *gc_obj);

     extern obj_thunk* get_deepest_obj_thunk(obj_thunk* obj);

     extern void create_static_class_data(ClazzFile* cf);

     extern void initialize_class(JNIEnv *env, ClazzFile* cf);

     extern obj_thunk *find_instance_data_for_class(obj_thunk *object_ref,
						    ClazzFile *cf);

     extern MethodStruct *get_interface_method_info(JNIEnv *env,
						    obj_thunk *obj,
						    ClazzFile **cf,
						    char *method_name,
						    Signature *sig);

     extern int is_instance_of(obj_thunk *obj, ClazzFile *cf);

#endif /* _objects_h */
