/* -*- Mode: C; c-file-style: "gnu" -*-
   operand_stack.h --
   Created: Chris Toshok <toshok@hungry.com>, 10-Aug-1997
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#ifndef _op_stack_h
#define _op_stack_h

#include "jni.h"
#include "sig.h"

typedef struct {
  SigPrimType tag;
  jvalue value;
} JavaStackItem;

typedef struct {
  JavaStackItem *stack_bottom;
  JavaStackItem *stack_top;
  JavaStackItem *high_water;
} OpStack;

#define OPSTACK_SIZE 16384 /* 16k operand stacks */

extern OpStack *op_stack_allocate(int size);
extern void op_stack_deallocate(OpStack *stack);

extern void op_stack_check_bottomspace(JNIEnv *env, OpStack *s,
				       int space_needed);
extern void op_stack_check_topspace(JNIEnv *env, OpStack *s,
				    int space_needed);

extern void op_stack_push_any(JNIEnv *env, OpStack *s, JavaStackItem *any);
extern void op_stack_push_value(JNIEnv *env, OpStack *s, SigPrimType tag, jvalue *value);
extern void op_stack_push_byte(JNIEnv *env, OpStack *s, jbyte value);
extern void op_stack_push_char(JNIEnv *env, OpStack *s, jchar value); /* really only 16 bit... */
extern void op_stack_push_short(JNIEnv *env, OpStack *s, jshort value);
extern void op_stack_push_boolean(JNIEnv *env, OpStack *s, jboolean value);
extern void op_stack_push_int(JNIEnv *env, OpStack *s, jint value);
extern void op_stack_push_long(JNIEnv *env, OpStack *s, jlong value);
extern void op_stack_push_long_w(JNIEnv *env, OpStack *s, jint word1, jint word2);
extern void op_stack_push_float(JNIEnv *env, OpStack *s, jfloat value);
extern void op_stack_push_double(JNIEnv *env, OpStack *s, jdouble value);
extern void op_stack_push_double_w(JNIEnv *env, OpStack *s, jint word1, jint word2);
extern void op_stack_push_object(JNIEnv *env, OpStack *s, jobject obj);

extern void op_stack_pop_any(JNIEnv *env, OpStack *s, JavaStackItem *any);
extern void op_stack_pop_value(JNIEnv *env, OpStack *s, jvalue *value);
extern void op_stack_pop_byte(JNIEnv *env, OpStack *s, jbyte *value);
extern void op_stack_pop_char(JNIEnv *env, OpStack *s, jchar *value); /* really only 16 bit... */
extern void op_stack_pop_short(JNIEnv *env, OpStack *s, jshort *value);
extern void op_stack_pop_boolean(JNIEnv *env, OpStack *s, jboolean *value);
extern void op_stack_pop_int(JNIEnv *env, OpStack *s, jint *value);
extern void op_stack_pop_long(JNIEnv *env, OpStack *s, jlong *value);
extern void op_stack_pop_long_w(JNIEnv *env, OpStack *s, jint *word1, jint *word2);
extern void op_stack_pop_float(JNIEnv *env, OpStack *s, jfloat *value);
extern void op_stack_pop_double(JNIEnv *env, OpStack *s, jdouble *value);
extern void op_stack_pop_double_w(JNIEnv *env, OpStack *s, jint *word1, jint *word2);
extern void op_stack_pop_object(JNIEnv *env, OpStack *s, jobject *obj);

extern int promote_to_int(JNIEnv *env, JavaStackItem i);

#endif /* _op_stack_h */
