/* -*- Mode: C; c-file-style: "gnu" -*-
   jni.h -- Java Native Interface header file.
   Created: Chris Toshok <toshok@hungry.com>, 26-Jul-1997
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#ifndef _hungry_jni_h
#define _hungry_jni_h

#include <stdio.h>
#include <sys/types.h>
#include <stdarg.h>

#include "jnitypes.h"

/* probably need to do something special for win32, but since there are
   so many other unixisms in this codebase, who gives a shit? */
#define JNIEXPORT
#define JNICALL

#define JNI_FALSE 0
#define JNI_TRUE 1

/* two constants used in Release<PrimitiveType>ArrayElements */
#define JNI_COMMIT 1
#define JNI_ABORT 2

/* the type used to describe cardinal indices and sizes */
typedef jint jsize;

/* JNI Reference types */
typedef jobject jclass;
typedef jobject jstring;
typedef jobject jarray;
typedef jarray jobjectArray;
typedef jarray jbooleanArray;
typedef jarray jbyteArray;
typedef jarray jcharArray;
typedef jarray jshortArray;
typedef jarray jintArray;
typedef jarray jlongArray;
typedef jarray jfloatArray;
typedef jarray jdoubleArray;
typedef jobject jthrowable;

/* JNI Field and Method ID's */
typedef struct FieldStruct *jfieldID;
typedef struct MethodStruct *jmethodID;

/* JNI Value Type */

typedef union jvalue {
  jboolean	z;
  jbyte		b;
  jchar		c;
  jshort	s;
  jint		i;
  jlong		j;
  jfloat	f;
  jdouble	d;
  jobject	l;
} jvalue;

/* Structure used when registering native methods. */
typedef struct {
  char *name;
  char *signature;
  void *fnPtr;
} JNINativeMethod;

typedef const struct JNIInvokeInterface *JavaVM;
typedef const struct JNINativeInterface *JNIEnv;

struct JNINativeInterface {
  void *_sun1;
  void *_sun2;
  void *_sun3;
  jint (*GetVersion)(JNIEnv *env);

  jclass (*DefineClass)(JNIEnv *env, jobject loader, const jbyte* buf, jsize bufLen);
  jclass (*FindClass)(JNIEnv *env, const char *name);

  void *_sun4;
  void *_sun5;
  void *_sun6;
  jclass (*GetSuperclass)(JNIEnv *env, jclass clazz);
  jboolean (*IsAssignableFrom)(JNIEnv *env, jclass clazz1, jclass clazz2);

  void *_sun7;

  jint (*Throw)(JNIEnv *env, jthrowable obj);
  jint (*ThrowNew)(JNIEnv *env, jclass clazz, const char *message);
  jthrowable (*ExceptionOccurred)(JNIEnv *env);
  void (*ExceptionDescribe)(JNIEnv *env);
  void (*ExceptionClear)(JNIEnv *env);
  void (*FatalError)(JNIEnv *env, const char *msg);
  void *_sun8;
  void *_sun9;

  jobject (*NewGlobalRef)(JNIEnv *env, jobject obj);
  void (*DeleteGlobalRef)(JNIEnv *env, jobject globalRef);
  void (*DeleteLocalRef)(JNIEnv *env, jobject localRef);

  jboolean (*IsSameObject)(JNIEnv *env, jobject ref, jobject ref2);
  void *_sun10;
  void *_sun11;
  
  jobject (*AllocObject)(JNIEnv *env, jclass clazz);
  jobject (*NewObject)(JNIEnv *env, jclass clazz,
		       jmethodID methodID, ...);
  jobject (*NewObjectV)(JNIEnv *env, jclass clazz,
			jmethodID methodID, va_list args);
  jobject (*NewObjectA)(JNIEnv *env, jclass clazz,
			jmethodID methodID, jvalue *args);

  jclass (*GetObjectClass)(JNIEnv *env, jobject obj);
  jboolean (*IsInstanceOf)(JNIEnv *env, jobject obj, jclass clazz);

  jmethodID (*GetMethodID)(JNIEnv *env, jclass clazz,
			   const char *name, const char *sig);

  jobject (*CallObjectMethod)(JNIEnv *env, jobject obj,
			      jmethodID methodID, ...);
  jobject (*CallObjectMethodV)(JNIEnv *env, jobject obj,
			       jmethodID methodID, va_list args);
  jobject (*CallObjectMethodA)(JNIEnv *env, jobject obj,
			       jmethodID methodID, jvalue *args);
  jboolean (*CallBooleanMethod)(JNIEnv *env, jobject obj,
				jmethodID methodID, ...);
  jboolean (*CallBooleanMethodV)(JNIEnv *env, jobject obj,
				 jmethodID methodID, va_list args);
  jboolean (*CallBooleanMethodA)(JNIEnv *env, jobject obj,
				 jmethodID methodID, jvalue *args);

  jbyte (*CallByteMethod)(JNIEnv *env, jobject obj,
			  jmethodID methodID, ...);
  jbyte (*CallByteMethodV)(JNIEnv *env, jobject obj,
			   jmethodID methodID, va_list args);
  jbyte (*CallByteMethodA)(JNIEnv *env, jobject obj,
			   jmethodID methodID, jvalue *args);

  jchar (*CallCharMethod)(JNIEnv *env, jobject obj,
			  jmethodID methodID, ...);
  jchar (*CallCharMethodV)(JNIEnv *env, jobject obj,
			   jmethodID methodID, va_list args);
  jchar (*CallCharMethodA)(JNIEnv *env, jobject obj,
			   jmethodID methodID, jvalue *args);

  jshort (*CallShortMethod)(JNIEnv *env, jobject obj,
			    jmethodID methodID, ...);
  jshort (*CallShortMethodV)(JNIEnv *env, jobject obj,
			     jmethodID methodID, va_list args);
  jshort (*CallShortMethodA)(JNIEnv *env, jobject obj,
			     jmethodID methodID, jvalue *args);

  jint (*CallIntMethod)(JNIEnv *env, jobject obj,
			jmethodID methodID, ...);
  jint (*CallIntMethodV)(JNIEnv *env, jobject obj,
			 jmethodID methodID, va_list args);
  jint (*CallIntMethodA)(JNIEnv *env, jobject obj,
			 jmethodID methodID, jvalue *args);

  jlong (*CallLongMethod)(JNIEnv *env, jobject obj,
			  jmethodID methodID, ...);
  jlong (*CallLongMethodV)(JNIEnv *env, jobject obj,
			   jmethodID methodID, va_list args);
  jlong (*CallLongMethodA)(JNIEnv *env, jobject obj,
			   jmethodID methodID, jvalue *args);

  jfloat (*CallFloatMethod)(JNIEnv *env, jobject obj,
			    jmethodID methodID, ...);
  jfloat (*CallFloatMethodV)(JNIEnv *env, jobject obj,
			     jmethodID methodID, va_list args);
  jfloat (*CallFloatMethodA)(JNIEnv *env, jobject obj,
			     jmethodID methodID, jvalue *args);

  jdouble (*CallDoubleMethod)(JNIEnv *env, jobject obj,
			      jmethodID methodID, ...);
  jdouble (*CallDoubleMethodV)(JNIEnv *env, jobject obj,
			       jmethodID methodID, va_list args);
  jdouble (*CallDoubleMethodA)(JNIEnv *env, jobject obj,
			       jmethodID methodID, jvalue *args);

  void (*CallVoidMethod)(JNIEnv *env, jobject obj,
			 jmethodID methodID, ...);
  void (*CallVoidMethodV)(JNIEnv *env, jobject obj,
			  jmethodID methodID, va_list args);
  void (*CallVoidMethodA)(JNIEnv *env, jobject obj,
			  jmethodID methodID, jvalue *args);


  jobject (*CallNonvirtualObjectMethod)(JNIEnv *env, jobject obj,
					jmethodID methodID, ...);
  jobject (*CallNonvirtualObjectMethodV)(JNIEnv *env, jobject obj,
					 jmethodID methodID, va_list args);
  jobject (*CallNonvirtualObjectMethodA)(JNIEnv *env, jobject obj,
					 jmethodID methodID, jvalue *args);
  jboolean (*CallNonvirtualBooleanMethod)(JNIEnv *env, jobject obj,
					  jmethodID methodID, ...);
  jboolean (*CallNonvirtualBooleanMethodV)(JNIEnv *env, jobject obj,
					   jmethodID methodID, va_list args);
  jboolean (*CallNonvirtualBooleanMethodA)(JNIEnv *env, jobject obj,
					   jmethodID methodID, jvalue *args);
  jbyte (*CallNonvirtualByteMethod)(JNIEnv *env, jobject obj,
				    jmethodID methodID, ...);
  jbyte (*CallNonvirtualByteMethodV)(JNIEnv *env, jobject obj,
				     jmethodID methodID, va_list args);
  jbyte (*CallNonvirtualByteMethodA)(JNIEnv *env, jobject obj,
				     jmethodID methodID, jvalue *args);
  jchar (*CallNonvirtualCharMethod)(JNIEnv *env, jobject obj,
				    jmethodID methodID, ...);
  jchar (*CallNonvirtualCharMethodV)(JNIEnv *env, jobject obj,
				     jmethodID methodID, va_list args);
  jchar (*CallNonvirtualCharMethodA)(JNIEnv *env, jobject obj,
				     jmethodID methodID, jvalue *args);
  jshort (*CallNonvirtualShortMethod)(JNIEnv *env, jobject obj,
				      jmethodID methodID, ...);
  jshort (*CallNonvirtualShortMethodV)(JNIEnv *env, jobject obj,
				       jmethodID methodID, va_list args);
  jshort (*CallNonvirtualShortMethodA)(JNIEnv *env, jobject obj,
				       jmethodID methodID, jvalue *args);
  jint (*CallNonvirtualIntMethod)(JNIEnv *env, jobject obj,
				  jmethodID methodID, ...);
  jint (*CallNonvirtualIntMethodV)(JNIEnv *env, jobject obj,
				   jmethodID methodID, va_list args);
  jint (*CallNonvirtualIntMethodA)(JNIEnv *env, jobject obj,
				   jmethodID methodID, jvalue *args);
  jlong (*CallNonvirtualLongMethod)(JNIEnv *env, jobject obj,
				    jmethodID methodID, ...);
  jlong (*CallNonvirtualLongMethodV)(JNIEnv *env, jobject obj,
				     jmethodID methodID, va_list args);
  jlong (*CallNonvirtualLongMethodA)(JNIEnv *env, jobject obj,
				     jmethodID methodID, jvalue *args);
  jfloat (*CallNonvirtualFloatMethod)(JNIEnv *env, jobject obj,
				      jmethodID methodID, ...);
  jfloat (*CallNonvirtualFloatMethodV)(JNIEnv *env, jobject obj,
				       jmethodID methodID, va_list args);
  jfloat (*CallNonvirtualFloatMethodA)(JNIEnv *env, jobject obj,
				       jmethodID methodID, jvalue *args);
  jdouble (*CallNonvirtualDoubleMethod)(JNIEnv *env, jobject obj,
					jmethodID methodID, ...);
  jdouble (*CallNonvirtualDoubleMethodV)(JNIEnv *env, jobject obj,
					 jmethodID methodID, va_list args);
  jdouble (*CallNonvirtualDoubleMethodA)(JNIEnv *env, jobject obj,
					 jmethodID methodID, jvalue *args);
  void (*CallNonvirtualVoidMethod)(JNIEnv *env, jobject obj,
				   jmethodID methodID, ...);
  void (*CallNonvirtualVoidMethodV)(JNIEnv *env, jobject obj,
				    jmethodID methodID, va_list args);
  void (*CallNonvirtualVoidMethodA)(JNIEnv *env, jobject obj,
				    jmethodID methodID, jvalue *args);

  jfieldID (*GetFieldID)(JNIEnv *env, jclass clazz,
			 const char *name, const char *sig);

  jobject (*GetObjectField)(JNIEnv *env, jobject obj,
			    jfieldID fieldID);

  jboolean (*GetBooleanField)(JNIEnv *env, jobject obj,
			      jfieldID fieldID);
  jbyte (*GetByteField)(JNIEnv *env, jobject obj,
			jfieldID fieldID);
  jchar (*GetCharField)(JNIEnv *env, jobject obj,
			jfieldID fieldID);
  jshort (*GetShortField)(JNIEnv *env, jobject obj,
			  jfieldID fieldID);
  jint (*GetIntField)(JNIEnv *env, jobject obj,
		      jfieldID fieldID);
  jlong (*GetLongField)(JNIEnv *env, jobject obj,
			jfieldID fieldID);
  jfloat (*GetFloatField)(JNIEnv *env, jobject obj,
			  jfieldID fieldID);
  jdouble (*GetDoubleField)(JNIEnv *env, jobject obj,
			    jfieldID fieldID);
  void (*SetObjectField)(JNIEnv *env, jobject obj,
			 jfieldID fieldID, jobject value);

  void (*SetBooleanField)(JNIEnv *env, jobject obj,
			  jfieldID fieldID, jboolean value);
  void (*SetByteField)(JNIEnv *env, jobject obj,
		       jfieldID fieldID, jbyte value);
  void (*SetCharField)(JNIEnv *env, jobject obj,
		       jfieldID fieldID, jchar value);
  void (*SetShortField)(JNIEnv *env, jobject obj,
			jfieldID fieldID, jshort value);
  void (*SetIntField)(JNIEnv *env, jobject obj,
		      jfieldID fieldID, jint value);
  void (*SetLongField)(JNIEnv *env, jobject obj,
		       jfieldID fieldID, jlong value);
  void (*SetFloatField)(JNIEnv *env, jobject obj,
			jfieldID fieldID, jfloat value);
  void (*SetDoubleField)(JNIEnv *env, jobject obj,
			 jfieldID fieldID, jdouble value);

  jmethodID (*GetStaticMethodID)(JNIEnv *env, jclass clazz,
				 const char *name, const char *sig);

  jobject (*CallStaticObjectMethod)(JNIEnv *env, jclass clazz,
				    jmethodID methodID, ...);
  jobject (*CallStaticObjectMethodV)(JNIEnv *env, jclass clazz,
				     jmethodID methodID, va_list args);
  jobject (*CallStaticObjectMethodA)(JNIEnv *env, jclass clazz,
				     jmethodID methodID, jvalue *args);

  jboolean (*CallStaticBooleanMethod)(JNIEnv *env, jclass clazz,
				      jmethodID methodID, ...);
  jboolean (*CallStaticBooleanMethodV)(JNIEnv *env, jclass clazz,
				       jmethodID methodID, va_list args);
  jboolean (*CallStaticBooleanMethodA)(JNIEnv *env, jclass clazz,
				       jmethodID methodID, jvalue *args);
  jbyte (*CallStaticByteMethod)(JNIEnv *env, jclass clazz,
				jmethodID methodID, ...);
  jbyte (*CallStaticByteMethodV)(JNIEnv *env, jclass clazz,
				 jmethodID methodID, va_list args);
  jbyte (*CallStaticByteMethodA)(JNIEnv *env, jclass clazz,
				 jmethodID methodID, jvalue *args);
  jchar (*CallStaticCharMethod)(JNIEnv *env, jclass clazz,
				jmethodID methodID, ...);
  jchar (*CallStaticCharMethodV)(JNIEnv *env, jclass clazz,
				 jmethodID methodID, va_list args);
  jchar (*CallStaticCharMethodA)(JNIEnv *env, jclass clazz,
				 jmethodID methodID, jvalue *args);
  jshort (*CallStaticShortMethod)(JNIEnv *env, jclass clazz,
				  jmethodID methodID, ...);
  jshort (*CallStaticShortMethodV)(JNIEnv *env, jclass clazz,
				   jmethodID methodID, va_list args);
  jshort (*CallStaticShortMethodA)(JNIEnv *env, jclass clazz,
				   jmethodID methodID, jvalue *args);
  jint (*CallStaticIntMethod)(JNIEnv *env, jclass clazz,
			      jmethodID methodID, ...);
  jint (*CallStaticIntMethodV)(JNIEnv *env, jclass clazz,
			       jmethodID methodID, va_list args);
  jint (*CallStaticIntMethodA)(JNIEnv *env, jclass clazz,
			       jmethodID methodID, jvalue *args);
  jlong (*CallStaticLongMethod)(JNIEnv *env, jclass clazz,
				jmethodID methodID, ...);
  jlong (*CallStaticLongMethodV)(JNIEnv *env, jclass clazz,
				 jmethodID methodID, va_list args);
  jlong (*CallStaticLongMethodA)(JNIEnv *env, jclass clazz,
				 jmethodID methodID, jvalue *args);
  jfloat (*CallStaticFloatMethod)(JNIEnv *env, jclass clazz,
				  jmethodID methodID, ...);
  jfloat (*CallStaticFloatMethodV)(JNIEnv *env, jclass clazz,
				   jmethodID methodID, va_list args);
  jfloat (*CallStaticFloatMethodA)(JNIEnv *env, jclass clazz,
				   jmethodID methodID, jvalue *args);
  jdouble (*CallStaticDoubleMethod)(JNIEnv *env, jclass clazz,
				    jmethodID methodID, ...);
  jdouble (*CallStaticDoubleMethodV)(JNIEnv *env, jclass clazz,
				     jmethodID methodID, va_list args);
  jdouble (*CallStaticDoubleMethodA)(JNIEnv *env, jclass clazz,
				     jmethodID methodID, jvalue *args);
  void (*CallStaticVoidMethod)(JNIEnv *env, jclass clazz,
			       jmethodID methodID, ...);
  void (*CallStaticVoidMethodV)(JNIEnv *env, jclass clazz,
				jmethodID methodID, va_list args);
  void (*CallStaticVoidMethodA)(JNIEnv *env, jclass clazz,
				jmethodID methodID, jvalue *args);

  jfieldID (*GetStaticFieldID)(JNIEnv *env, jclass clazz,
			       const char *name, const char *sig);

  jobject (*GetStaticObjectField)(JNIEnv *env, jclass clazz,
				  jfieldID fieldID);
  jboolean (*GetStaticBooleanField)(JNIEnv *env, jclass clazz,
				    jfieldID fieldID);
  jbyte (*GetStaticByteField)(JNIEnv *env, jclass clazz,
			      jfieldID fieldID);
  jchar (*GetStaticCharField)(JNIEnv *env, jclass clazz,
			      jfieldID fieldID);
  jshort (*GetStaticShortField)(JNIEnv *env, jclass clazz,
				jfieldID fieldID);
  jint (*GetStaticIntField)(JNIEnv *env, jclass clazz,
			    jfieldID fieldID);
  jlong (*GetStaticLongField)(JNIEnv *env, jclass clazz,
			      jfieldID fieldID);
  jfloat (*GetStaticFloatField)(JNIEnv *env, jclass clazz,
				jfieldID fieldID);
  jdouble (*GetStaticDoubleField)(JNIEnv *env, jclass clazz,
				  jfieldID fieldID);

  void (*SetStaticObjectField)(JNIEnv *env, jclass clazz,
			       jfieldID fieldID, jobject value);
  void (*SetStaticBooleanField)(JNIEnv *env, jclass clazz,
				jfieldID fieldID, jboolean value);
  void (*SetStaticByteField)(JNIEnv *env, jclass clazz,
			     jfieldID fieldID, jbyte value);
  void (*SetStaticCharField)(JNIEnv *env, jclass clazz,
			     jfieldID fieldID, jchar value);
  void (*SetStaticShortField)(JNIEnv *env, jclass clazz,
			      jfieldID fieldID, jshort value);
  void (*SetStaticIntField)(JNIEnv *env, jclass clazz,
			    jfieldID fieldID, jint value);
  void (*SetStaticLongField)(JNIEnv *env, jclass clazz,
			     jfieldID fieldID, jlong value);
  void (*SetStaticFloatField)(JNIEnv *env, jclass clazz,
			      jfieldID fieldID, jfloat value);
  void (*SetStaticDoubleField)(JNIEnv *env, jclass clazz,
			       jfieldID fieldID, jdouble value);


  jstring (*NewString)(JNIEnv *env, const jchar *unicodeChars, jsize len);
  jsize (*GetStringLength)(JNIEnv *env, jstring string);
  jchar* (*GetStringChars)(JNIEnv *env, jstring string, jboolean *isCopy);
  void (*ReleaseStringChars)(JNIEnv *env, jstring string, const jchar *chars);

  jstring (*NewStringUTF)(JNIEnv *env, const char *bytes);
  jsize (*GetStringUTFLength)(JNIEnv *env, jstring string);
  const jbyte* (*GetStringUTFChars)(JNIEnv *env, jstring string, jboolean *isCopy);
  void (*ReleaseStringUTFChars)(JNIEnv *env, jstring string, const jbyte *utf);

  jsize (*GetArrayLength)(JNIEnv *env, jarray array);

  jarray (*NewObjectArray)(JNIEnv *env, jsize length,
			   jclass elementClass, jobject initialElement);
  jobject (*GetObjectArrayElement)(JNIEnv *env, jobjectArray array, jsize index);
  void (*SetObjectArrayElement)(JNIEnv *env, jobjectArray array,
				jsize index, jobject value);

  jbooleanArray (*NewBooleanArray)(JNIEnv *env, jsize length);
  jbyteArray (*NewByteArray)(JNIEnv *env, jsize length);
  jcharArray (*NewCharArray)(JNIEnv *env, jsize length);
  jshortArray (*NewShortArray)(JNIEnv *env, jsize length);
  jintArray (*NewIntArray)(JNIEnv *env, jsize length);
  jlongArray (*NewLongArray)(JNIEnv *env, jsize length);
  jfloatArray (*NewFloatArray)(JNIEnv *env, jsize length);
  jdoubleArray (*NewDoubleArray)(JNIEnv *env, jsize length);

  jboolean* (*GetBooleanArrayElements)(JNIEnv *env, jbooleanArray array,
				       jboolean* isCopy);
  jbyte* (*GetByteArrayElements)(JNIEnv *env, jbyteArray array,
				 jboolean* isCopy);
  jchar* (*GetCharArrayElements)(JNIEnv *env, jcharArray array,
				 jboolean* isCopy);
  jshort* (*GetShortArrayElements)(JNIEnv *env, jshortArray array,
				   jboolean* isCopy);
  jint* (*GetIntArrayElements)(JNIEnv *env, jintArray array,
			       jboolean* isCopy);
  jlong* (*GetLongArrayElements)(JNIEnv *env, jlongArray array,
				 jboolean* isCopy);
  jfloat* (*GetFloatArrayElements)(JNIEnv *env, jfloatArray array,
				   jboolean* isCopy);
  jdouble* (*GetDoubleArrayElements)(JNIEnv *env, jdoubleArray array,
				     jboolean* isCopy);

  void (*ReleaseBooleanArrayElements)(JNIEnv *env, jbooleanArray array,
				      jboolean* elems, jint mode);
  void (*ReleaseByteArrayElements)(JNIEnv *env, jbyteArray array,
				   jbyte* elems, jint mode);
  void (*ReleaseCharArrayElements)(JNIEnv *env, jcharArray array,
				   jchar* elems, jint mode);
  void (*ReleaseShortArrayElements)(JNIEnv *env, jshortArray array,
				    jshort* elems, jint mode);
  void (*ReleaseIntArrayElements)(JNIEnv *env, jintArray array,
				  jint* elems, jint mode);
  void (*ReleaseLongArrayElements)(JNIEnv *env, jlongArray array,
				   jlong* elems, jint mode);
  void (*ReleaseFloatArrayElements)(JNIEnv *env, jfloatArray array,
				    jfloat* elems, jint mode);
  void (*ReleaseDoubleArrayElements)(JNIEnv *env, jdoubleArray array,
				     jdouble* elems, jint mode);

  void (*GetBooleanArrayRegion)(JNIEnv *env, jbooleanArray array,
				jsize start, jsize len, jboolean* buf);
  void (*GetByteArrayRegion)(JNIEnv *env, jbyteArray array,
			     jsize start, jsize len, jbyte* buf);
  void (*GetCharArrayRegion)(JNIEnv *env, jcharArray array,
			     jsize start, jsize len, jchar* buf);
  void (*GetShortArrayRegion)(JNIEnv *env, jshortArray array,
			      jsize start, jsize len, jshort* buf);
  void (*GetIntArrayRegion)(JNIEnv *env, jintArray array,
			    jsize start, jsize len, jint* buf);
  void (*GetLongArrayRegion)(JNIEnv *env, jlongArray array,
			     jsize start, jsize len, jlong* buf);
  void (*GetFloatArrayRegion)(JNIEnv *env, jfloatArray array,
			      jsize start, jsize len, jfloat* buf);
  void (*GetDoubleArrayRegion)(JNIEnv *env, jdoubleArray array,
			       jsize start, jsize len, jdouble* buf);
  
  jint (*RegisterNatives)(JNIEnv *env, jclass clazz,
			  const JNINativeMethod *methods, jint nMethods);
  jint (*UnregisterNatives)(JNIEnv *env, jclass clazz,
			    const JNINativeMethod *methods, jint nMethods);

  jint (*MonitorEnter)(JNIEnv *env, jobject obj);
  jint (*MonitorExit)(JNIEnv *env, jobject obj);

  jint (*GetJavaVM)(JNIEnv *env, JavaVM **vm);
};

/* Types needed for the JNI Invocation API */
typedef struct JavaVMInitArgs {
  /*
  ** Java VM version
  */
  jint version;

  /*
  ** System properties
  */
  char **properties;

  /* 
  ** whether to check the java source files are newer than
  ** compiled class files 
  */
  jint checkSource;

  /*
  ** maximum native stack size of Java-created threads
  */
  jint nativeStackSize;

  /*
  ** maximum java stack size.
  */
  jint javaStackSize;
  
  /*
  ** initial heap size.
  */
  jint minHeapSize;

  /*
  ** maximum heap size.
  */
  jint maxHeapSize;

  /*
  ** controls whether java byte code should be verified;
  ** 0 -- none, 1 -- remotely loaded code, 2 -- all code.
  */
  jint verifyMode;

  /*
  ** the local directory path for class loading.
  */
  const char *classpath;
  
  /*
  ** a hook for a function that redirects all VM messages.
  */
  jint (*vfprintf)(FILE *fp, const char *format,
		   va_list args);

  /*
  ** a VM exit hook.
  */
  void (*exit)(jint code);

  /*
  ** a VM abort hook.
  */
  void (*abort)();

  /*
  ** whether to enable class GC.
  */
  jint enableClassGC;

  /*
  ** whether GC messages will appear.
  */
  jint enableVerboseGC;

  /*
  ** whether asynchronous GC is allowed.
  */
  jint disableAsyncGC;

  /*
  ** Three reserved fields.
  */
  jint reserved0;
  jint reserved1;
  /* XXX hungry added field... */
  jboolean enableVerboseClassLoading;
  /*  jint reserved2;*/
} JDK1_1InitArgs;

typedef struct JDK1_1AttachArgs {
  /*
  ** JDK 1.1 does not need any arguments to attach a native
  ** thread.  The padding is here to satisfy the C compiler
  ** which does not permit empty structures.
  */
  void *__mumble_padding;
} JDK1_1AttachArgs;

struct JNIInvokeInterface {
  void *_sun1;
  void *_sun2;
  void *_sun3;
  jint (*DestroyJavaVM)(JavaVM *vm);
  jint (*AttachCurrentThread)(JavaVM *vm, JNIEnv **p_env, void *thr_args);
  jint (*DetachCurrentThread)(JavaVM *vm);
};

extern jint JNI_GetDefaultJavaVMInitArgs(void *vm_args);
extern jint JNI_GetCreatedJavaVMs(JavaVM **vmBuf, jsize bufLen,
				  jsize *nVMs);
extern jint JNI_CreateJavaVM(JavaVM **pMvm, JNIEnv **p_env,
			     void *vm_args);

#endif /* _hungry_jni_h */

