/* -*- Mode: C; c-file-style: "gnu" -*-
   jniclass.c -- Java Native Interface methods relating to classes.
   Created: Chris Toshok <toshok@hungry.com>, 26-Jul-1997
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#include "jni.h"
#include "jniint.h"
#include "objects.h"
#include "ClazzFile.h"
#include "resolve.h"

#include <string.h>

jclass
JNIFUNC(DefineClass)(JNIEnv *env,
		     jobject loader,
		     const jbyte* buf,
		     jsize bufLen)
{
  jclass cls = define_class(env,(char*)buf, bufLen);

  /* check for exceptions? */
  return cls;
}

jclass
JNIFUNC(FindClass)(JNIEnv *env,
		   const char *name)
{
  jclass cls = NULL;
  char *foo_name = strdup(name);

  if (foo_name[0] == 'L' && foo_name[strlen(foo_name) - 1] == ';')
    {
      foo_name++;
      foo_name[strlen(foo_name) - 1] = 0;
    }

  cls = find_class(env, foo_name);

  if (!cls)
    {
      jclass class_not_found = NULL;
      /* XXX avoid endless recursion if ClassNotFoundException is not found */
      if (0 == strcmp(foo_name, "java/lang/ClassNotFoundException"))
	return NULL;

      class_not_found = (*env)->FindClass(env, "java/lang/ClassNotFoundException");
      if (class_not_found) /* Avoid calling class if it was not found */
	(*env)->ThrowNew(env, class_not_found, foo_name);

      return NULL;
    }
  else
    return cls;
}

jclass
JNIFUNC(GetSuperclass)(JNIEnv *env,
		       jclass clazz)
{
  ClazzFile *clazzfile = (ClazzFile*)clazz;
  /* if clazz specifies the class Object, or clazz represents an interface, this
  ** function returns NULL
  */

  if (clazzfile->access_flags & ACC_INTERFACE
      || clazzfile->superclass == NULL)
    return NULL;
  else
    return clazzfile->superclass;
}

jboolean
JNIFUNC(IsAssignableFrom)(JNIEnv *env,
			  jclass clazz1,
			  jclass clazz2)
{
  int i;
  ClazzFile *clazzfile1 = (ClazzFile*)clazz1;
  ClazzFile *clazzfile2 = (ClazzFile*)clazz2;
  ClazzFile *clazz;

  clazz = clazzfile2;

  /* if clazzfile1 exists as an ancestor of clazz, or if it is one
     of the interfaces of an ancestor, return JNI_TRUE. */
  do {
    if (clazz == clazzfile1)
      return JNI_TRUE;
    else
      {
	for (i = 0; i < clazz->num_interfaces; i ++)
	  if (getInterface(env, clazz, i) == clazzfile1)
	    return JNI_TRUE;
      }
    clazz = getSuperClass(env, clazz);
  } while (clazz);

  return JNI_FALSE;
}
