/* -*- Mode: C; c-file-style: "gnu" -*-
   jnifield.c -- Java Native Interface methods relating to object fields.
   Created: Chris Toshok <toshok@hungry.com>, 26-Jul-1997
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#include "jni.h"
#include "jniint.h"
#include "ClazzFile.h"
#include "objects.h"
#include "interp.h"
#include <assert.h>

jfieldID
JNIFUNC(GetFieldID)(JNIEnv *env,
		    jclass clazz,
		    const char *name,
		    const char *sig)
{
  FieldStruct *field = NULL;
  ClazzFile *cf = (ClazzFile*)clazz;
  Signature *field_sig;
  int field_num;

  assert(NULL != env);
  assert(NULL != clazz);
  assert(NULL != cf);
  assert(NULL != name);
  assert(NULL != sig);

  /* XXX initialize the class, if needed, throwing ExceptionInInitializerError
     if there was an error. */

  field_sig = SIG_parseFromJavaSig(env, (char*)sig);
  if (field_sig == NULL)
    {
      jclass out_of_memory = (*env)->FindClass(env, "java/lang/OutOfMemoryError");
      if (out_of_memory)
	(*env)->ThrowNew(env, out_of_memory, name);
      else
	(*env)->FatalError(env, "Could not load java/lang/NoSuchFieldException");
      return NULL;
    }

  for (field_num = 0; field_num < cf->num_fields; field_num++)
    {
      FieldStruct *cur_field = cf->fields[field_num];

      if (cur_field->access_flags & ACC_STATIC)
	continue;

      if (!strcmp(cur_field->name, name)
	  && SIG_equal(env, field_sig, cur_field->sig))
	{
	  field = cur_field;
	  break;
	}
    }
  
  SIG_free(env, field_sig);
  
  if (field == NULL)
    {
      jclass no_such_field = (*env)->FindClass(env, "java/lang/NoSuchFieldException");
      if (no_such_field)
	(*env)->ThrowNew(env, no_such_field, name);
      else
	(*env)->FatalError(env, "Could not load java/lang/NoSuchFieldException");
      return NULL;
    }
  
  return field;
}

#define GETSET_FIELD(type, pretty_type, sig_type, union_element) \
type \
JNIFUNC(Get##pretty_type##Field)(JNIEnv *env, \
				 jobject obj, \
				 jfieldID fieldID) \
{ \
    FieldStruct *field = (FieldStruct*)fieldID; \
						  JavaStackItem item; \
									\
									obj = (jobject)find_instance_data_for_class(obj, field->clazz);\
																	 get_instance_field(obj, field, &item); \
																						  \
																						  return item.value.union_element; \
																										     }  \
void \
																										     JNIFUNC(Set##pretty_type##Field)(JNIEnv *env, \
																														      jobject obj, \
																														      jfieldID fieldID, \
																														      type value) \
{ \
    FieldStruct *field = (FieldStruct*)fieldID; \
						  JavaStackItem item; \
									\
									item.tag = sig_type; \
											       item.value.union_element = value; \
																   \
																   obj = (jobject)find_instance_data_for_class(obj, field->clazz);\
																								    set_instance_field(obj, field, item);  \
																													     }


GETSET_FIELD(jboolean, Boolean, SIG_JBOOLEAN, z)
     GETSET_FIELD(jbyte, Byte, SIG_JBYTE, b)
     GETSET_FIELD(jchar, Char, SIG_JCHAR, c)
     GETSET_FIELD(jshort, Short, SIG_JSHORT, s)
     GETSET_FIELD(jint, Int, SIG_JINT, i)
     GETSET_FIELD(jlong, Long, SIG_JLONG, j)
     GETSET_FIELD(jfloat, Float, SIG_JFLOAT, f)
     GETSET_FIELD(jdouble, Double, SIG_JDOUBLE, d)
     GETSET_FIELD(jobject, Object, SIG_JOBJECT, l)
