/* -*- Mode: C; c-file-style: "gnu" -*-
   jnistr.c -- Java Native Interface string operations.
   Created: Chris Toshok <toshok@hungry.com>, 26-Jul-1997
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#include "jni.h"
#include "jniint.h"
#include <stdlib.h>
#include <string.h>

jstring
JNIFUNC(NewString)(JNIEnv *env,
		   const jchar *unicodeChars,
		   jsize len)
{
  return NULL;
}

jsize
JNIFUNC(GetStringLength)(JNIEnv *env,
			 jstring string)
{
  return 0;
}

jchar*
JNIFUNC(GetStringChars)(JNIEnv *env,
			jstring string,
			jboolean *isCopy)
{
  return NULL;
}

void
JNIFUNC(ReleaseStringChars)(JNIEnv *env,
			    jstring string,
			    const jchar *chars)
{
}

jstring
JNIFUNC(NewStringUTF)(JNIEnv *env,
		      const char *bytes)
{
#ifndef I_WANT_TO_CRASH
  HungryJNIEnv *henv = (HungryJNIEnv*)env;
  static jclass jstring_class = NULL;
  jstring new_string;
  jmethodID constructor;
  jcharArray char_array;
  jchar* array_elements;
  int length = strlen(bytes);
  int i;

  if (!jstring_class)
    {
      jstring_class = (*env)->FindClass(env, "java/lang/String");
      GC_addRoot((JavaVM*)henv->_vm, (GC_obj*)jstring_class);
    }

  /*  printf ("NewStringUTF(%s)\n", bytes);*/

  char_array = (*env)->NewCharArray(env, length);

  char_array = (*env)->NewGlobalRef(env, char_array);

  array_elements = (*env)->GetCharArrayElements(env, char_array, NULL);
  for (i = 0; i < length; i ++)
    array_elements[i] = bytes[i];

  (*env)->ReleaseCharArrayElements(env, char_array, array_elements, 0);

  constructor = (*env)->GetMethodID(env, jstring_class, "<init>", "([C)V");

  new_string = (*env)->NewObject(env, jstring_class, constructor, char_array, 0, length);

  return new_string;
#else
  static jclass jstring_class = NULL;
  jstring new_string;
  jmethodID constructor;
  jbyteArray byte_array;
  jbyte* array_elements;
  jboolean isCopy = JNI_FALSE;
  int length = strlen(bytes);

  if (!jstring_class)
    {
      jstring_class = (*env)->FindClass(env, "java/lang/String");
#if 0
      GC_addRoot((GC_obj*)jstring_class);
#endif
    }

  /*  printf ("NewStringUTF(%s)\n", bytes);*/

  byte_array = (*env)->NewByteArray(env, length);
  
  array_elements = (*env)->GetByteArrayElements(env, byte_array, &isCopy);
  memcpy(array_elements, bytes, length);

  constructor = (*env)->GetMethodID(env, jstring_class, "<init>", "([BII)V");

  new_string = (*env)->NewObject(env, jstring_class, constructor, byte_array, 0, length);

  return new_string;
#endif
}

jsize
JNIFUNC(GetStringUTFLength)(JNIEnv *env,
			    jstring string)
{
  jsize length;
  jclass string_class = (*env)->FindClass(env, "java/lang/String");
  jfieldID length_field = (*env)->GetFieldID(env,
					     string_class, "count", "I");
  length = (*env)->GetIntField(env, string, length_field);

  return length;
}

const jbyte*
JNIFUNC(GetStringUTFChars)(JNIEnv *env,
			   jstring string,
			   jboolean *isCopy)
{
  jsize length = (*env)->GetStringUTFLength(env, string);
  jbyte *bytes = (jbyte*)calloc(length, sizeof(bytes));
  jclass string_class = (*env)->FindClass(env, "java/lang/String");
  jfieldID value_field = (*env)->GetFieldID(env, string_class, "value", "[C");
  jfieldID offset_field = (*env)->GetFieldID(env, string_class, "offset", "I");
  jint offset = (*env)->GetIntField(env, string, offset_field);
  jcharArray char_array = (*env)->GetObjectField(env, string, value_field);
  jboolean char_copy;
  jchar* chars = (*env)->GetCharArrayElements(env, char_array, &char_copy);
  int i;

  if (isCopy)
    *isCopy = JNI_TRUE;

  for (i = 0; i < length; i ++)
    bytes[i] = chars[i + offset] & 0xff; /* lop off the upper 8 bits */

  (*env)->ReleaseCharArrayElements(env, char_array, chars, 0);

  return bytes;
}

void
JNIFUNC(ReleaseStringUTFChars)(JNIEnv *env,
			       jstring string,
			       const jbyte *utf)
{
  free((jbyte*)utf);
}

