/* -*- Mode: C; c-file-style: "gnu" -*-
   jvmdi.h -- Java Virtual Machine Debugger Interface (JDK 1.2)
   Created: Chris Toshok <toshok@hungry.com>, 9-Feb-1998.
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#ifndef _jvmdi_h
#define _jvmdi_h

#include "jni.h"

/* version number = 1.0 */
#define JVMDI_VERSION_NUMBER 0x00010000

typedef void* jthread;
typedef void* jframeID;

typedef enum {
  JVMDI_ERROR_NONE,
  JVMDI_ERROR_NULL_POINTER,
  JVMDI_ERROR_OUT_OF_MEMORY,
  JVMDI_ERROR_INVALID_METHODID,
  JVMDI_ERROR_INVALID_CLASS,
  JVMDI_ERROR_INVALID_BCI,
  JVMDI_ERROR_NO_SUCH_BREAKPOINT,
  JVMDI_ERROR_VM_DEAD,
  JVMDI_ERROR_INVALID_FRAMEID,
  JVMDI_ERROR_INVALID_SLOT,
  JVMDI_ERROR_TYPE_MISMATCH,
  JVMDI_ERROR_NATIVE_FRAME,
  JVMDI_ERROR_NO_MORE_FRAMES,
  JVMDI_ERROR_INVALID_THREAD,
  JVMDI_ERROR_THREAD_NOT_SUSPENDED,
} jvmdiError;

/* thread statuses */
enum {
  JVMDI_THREAD_STATUS_UNKNOWN,
  JVMDI_THREAD_STATUS_ZOMBIE,
  JVMDI_THREAD_STATUS_RUNNING,
  JVMDI_THREAD_STATUS_SLEEPING,
  JVMDI_THREAD_STATUS_MONITOR,
  JVMDI_THREAD_STATUS_WAIT,
  JVMDI_THREAD_STATUS_SUSPENDED,
  JVMDI_THREAD_STATUS_BREAK
};

/* for the "kind" file in JVMDI_Event */
enum {
  JVMDI_EVENT_SINGLE_STEP,
  JVMDI_EVENT_BREAKPOINT,
  JVMDI_EVENT_FRAME_POP,
  JVMDI_EVENT_EXCEPTION,
  JVMDI_EVENT_USER_DEFINED,
  JVMDI_EVENT_THREAD_END,
  JVMDI_EVENT_THREAD_START,
  JVMDI_EVENT_VM_DEATH
};

typedef struct {
  jint kind;                /* the discriminant */
  
  union {
    struct { /* kind = JVMDI_EVENT_SINGLE_STEP */
      jthread thread;
      jclass clazz;
      jmethodID method;
      jint bci;
    } single_step;
    
    struct { /* kind = JVMDI_EVENT_BREAKPOINT */
      jthread thread;
      jclass clazz;
      jmethodID method;
      jint bci;
    } breakpoint;
    
    struct { /* kind = JVMDI_EVENT_FRAME_POP */
      jthread thread;
      jframeID frame;
    } frame_pop;
    
    struct { /* kind = JVMDI_EVENT_EXCEPTION */
      jthread thread;
      jclass clazz;
      jmethodID method;
      jint bci;
      jobject exception;
      jclass catch_clazz;
      jmethodID catch_method;
      jint catch_bci;
    } exception;
    
    struct { /* kind = JVMDI_EVENT_USER_DEFINED */
      jobject object;
      jint key;
    } user;
    
    struct { /* kind = JVMDI_EVENT_THREAD_END or */
      /* JVMDI_EVENT_THREAD_START */
      jthread thread;
    } thread_change;
    
    /* kind = JVMDI_EVENT_VM_DEATH */
    /* no additional fields */                
  } u;
} JVMDI_Event;

JNIEXPORT jvmdiError JNICALL
JVMDI_GetThreadStatus(JNIEnv *env, jthread thread, jint *statusPtr);

JNIEXPORT jvmdiError JNICALL
JVMDI_SuspendThread(JNIEnv *env, jthread thread); 

JNIEXPORT jvmdiError JNICALL
JVMDI_ResumeThread(JNIEnv *env, jthread thread);

JNIEXPORT jvmdiError JNICALL
JVMDI_SetSingleStep(JNIEnv *env, jthread thread, jboolean shouldStep);

JNIEXPORT jvmdiError JNICALL
JVMDI_GetCurrentFrame(JNIEnv *env, jthread thread, jframeID *framePtr);

JNIEXPORT jvmdiError JNICALL
JVMDI_GetCallerFrame(JNIEnv *env, jframeID called, jframeID *framePtr);

JNIEXPORT jvmdiError JNICALL
JVMDI_GetFrameMethod(JNIEnv *env, jframeID frame, 
		     jclass *classPtr, jmethodID *methodPtr);

JNIEXPORT jvmdiError JNICALL
JVMDI_GetFrameBCI(JNIEnv *env, jframeID frame, jint *bciPtr);

JNIEXPORT jvmdiError JNICALL
JVMDI_NotifyFramePop(JNIEnv *env, jframeID frame); 

JNIEXPORT jvmdiError JNICALL
JVMDI_GetLocalObject(JNIEnv *env, jframeID frame, jint slot, 
		     jobject *valuePtr);

JNIEXPORT jvmdiError JNICALL
JVMDI_GetLocalInt(JNIEnv *env, jframeID frame, jint slot, 
		  jint *valuePtr);

JNIEXPORT jvmdiError JNICALL
JVMDI_GetLocalLong(JNIEnv *env, jframeID frame, jint slot, 
		   jlong *valuePtr);

JNIEXPORT jvmdiError JNICALL
JVMDI_GetLocalFloat(JNIEnv *env, jframeID frame, jint slot, 
		    jfloat *valuePtr);

JNIEXPORT jvmdiError JNICALL
JVMDI_GetLocalDouble(JNIEnv *env, jframeID frame, jint slot, 
		     jdouble *valuePtr);

JNIEXPORT jvmdiError JNICALL
JVMDI_SetLocalObject(JNIEnv *env, jframeID frame, jint slot, jobject value);

JNIEXPORT jvmdiError JNICALL
JVMDI_SetLocalInt(JNIEnv *env, jframeID frame, jint slot, jint value);

JNIEXPORT jvmdiError JNICALL
JVMDI_SetLocalFloat(JNIEnv *env, jframeID frame, jint slot, jfloat value);

JNIEXPORT jvmdiError JNICALL
JVMDI_SetLocalLong(JNIEnv *env, jframeID frame, jint slot, jlong value);

JNIEXPORT jvmdiError JNICALL
JVMDI_SetLocalDouble(JNIEnv *env, jframeID frame, jint slot, jdouble value);

JNIEXPORT jvmdiError JNICALL
JVMDI_SetBreakpoint(JNIEnv *env, jclass clazz, jmethodID method, jint bci);

JNIEXPORT jvmdiError JNICALL
JVMDI_ClearBreakpoint(JNIEnv *env, jclass clazz, jmethodID method, jint bci);

JNIEXPORT jvmdiError JNICALL
JVMDI_ClearAllBreakpoints(JNIEnv *env);

JNIEXPORT jvmdiError JNICALL
JVMDI_GetMethodName(JNIEnv *env, jclass clazz, jmethodID method, 
		    jstring *namePtr, jstring *signaturePtr);

JNIEXPORT jvmdiError JNICALL
JVMDI_GetMethodDefiningClass(JNIEnv *env, jclass clazz, jmethodID method,
			     jclass *definingClassPtr);

JNIEXPORT jvmdiError JNICALL
JVMDI_IsMethodNative(JNIEnv *env, jclass clazz, jmethodID method, 
		     jboolean *isNativePtr);

typedef void (*JVMDI_EventHook)(JNIEnv *env, JVMDI_Event *event);

JNIEXPORT jvmdiError JNICALL
JVMDI_SetEventHook(JNIEnv *env, JVMDI_EventHook hook);

JNIEXPORT jvmdiError JNICALL
JVMDI_GetLoadedClasses(JNIEnv *env, jobjectArray *classesPtr);

JNIEXPORT jvmdiError JNICALL
JVMDI_GetVersionNumber(JNIEnv *env, jint *versionPtr);

#endif _jvmdi_h
