/* -*- Mode: C; c-file-style: "gnu" -*-
   thread.c -- native methods for java/lang/Thread.
   Created: Chris Toshok <toshok@hungry.com>, 28-Jul-1997
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#include "jni.h"
#include "interp.h"
#include "native-threads.h"
#include "exceptions.h"
#include "log.h"

#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>

#define MYLOG "Native"

JNIEXPORT jobject JNICALL
Java_java_lang_Thread_currentThread(JNIEnv *env,
				    jclass cls)
{
  JThreadInfo *info = THREAD_getJavaInfo();

  return info->java_thread;
}

JNIEXPORT void JNICALL
Java_java_lang_Thread_yield(JNIEnv *env,
			    jclass cls)
{
  /* XXX more here? */
  THREAD_yield();
}

JNIEXPORT void JNICALL
Java_java_lang_Thread_sleep(JNIEnv *env,
			    jclass cls,
			    jlong millis)
{
  struct timeval timeout;

  timeout.tv_sec = millis / 1000;
  timeout.tv_usec = millis % 1000;

  select(0, NULL, NULL, NULL, &timeout);
}

static void *
real_thread_start(void *foo)
{
  jmethodID run_method;
  jobject obj = (jobject)foo;
  JNIEnv *env = THREAD_getEnv();
  JThreadInfo *native_thread_info;

  jclass thread_class = (*env)->GetObjectClass(env, obj);

  native_thread_info = (JThreadInfo*)calloc(1, sizeof(JThreadInfo));
  if (!native_thread_info)
    {
      throw_Exception(env, "java/lang/VirtualMachineError",
		      "Cannot allocate thread-local stack");
      return NULL;
    }
  native_thread_info->java_thread = obj;
  native_thread_info->name = strdup("java-thread");
  native_thread_info->op_stack = op_stack_allocate(OPSTACK_SIZE);
  native_thread_info->alive = JNI_TRUE;

  native_thread_info->stack_lowwater = (StackFrame*)malloc(sizeof(StackFrame) * 1000);
  native_thread_info->stack_highwater =	native_thread_info->stack_lowwater;
  native_thread_info->stack_highwater += 1000;
  native_thread_info->top_frame = native_thread_info->stack_highwater;

  THREAD_setJavaInfo(native_thread_info);

  run_method = (*env)->GetMethodID(env, thread_class, "run", "()V");

  printf ("run object = %p\n", obj);

  (*env)->CallVoidMethod(env, obj, run_method);

  printf("run object = %p done\n", obj);

  native_thread_info->alive = JNI_FALSE;

  return NULL;
}

JNIEXPORT void JNICALL
Java_java_lang_Thread_start(JNIEnv *env,
			    jobject obj)
{
  jclass thread_class = (*env)->FindClass(env, "java/lang/Thread");
  jfieldID priorityfield = (*env)->GetFieldID(env, thread_class, "priority", "I");
  jint priority = (*env)->GetIntField(env, obj, priorityfield);

  THREAD_start(real_thread_start,
	       obj,
	       priority,
	       16000 /* XXX initialStackSize */);
}

JNIEXPORT jboolean JNICALL
Java_java_lang_Thread_isInterrupted(JNIEnv *env,
				    jobject obj,
				    jboolean ClearInterrupted)
{
  (*env)->FatalError(env, "java.lang.Thread.isInterrupted() unimplemented");
  return JNI_FALSE;
}

JNIEXPORT jboolean JNICALL
Java_java_lang_Thread_isAlive(JNIEnv *env,
			      jobject obj)
{
  JThreadInfo *info = THREAD_getJavaInfo();

  return (NULL != info) ? info->alive : JNI_FALSE;
}

JNIEXPORT jint JNICALL
Java_java_lang_Thread_countStackFrames(JNIEnv *env,
				       jobject obj)
{
  (*env)->FatalError(env, "java.lang.Thread.countStackFrames() unimplemented");
  return 0;
}

JNIEXPORT void JNICALL
Java_java_lang_Thread_setPriority0(JNIEnv *env,
				   jobject obj,
				   jint newPriority)
{
  jclass thread_class = (*env)->FindClass(env, "java/lang/Thread");
  jfieldID priority = (*env)->GetFieldID(env, thread_class, "priority", "I");

  (*env)->SetIntField(env, obj, priority, newPriority);
  /* XXX Should also change thread priority */
}

JNIEXPORT void JNICALL
Java_java_lang_Thread_stop0(JNIEnv *env,
			    jobject obj,
			    jobject o)
{
  (*env)->FatalError(env, "java.lang.Thread.stop0() unimplemented");
}

JNIEXPORT void JNICALL
Java_java_lang_Thread_suspend0(JNIEnv *env,
			       jobject obj)
{
  (*env)->FatalError(env, "java.lang.Thread.suspend0() unimplemented");
}

JNIEXPORT void JNICALL
Java_java_lang_Thread_resume0(JNIEnv *env,
			      jobject obj)
{
  (*env)->FatalError(env, "java.lang.Thread.resume0() unimplemented");
}

JNIEXPORT void JNICALL
Java_java_lang_Thread_interrupt0(JNIEnv *env,
				 jobject obj)
{
  (*env)->FatalError(env, "java.lang.Thread.interrupt0() unimplemented");
}

