/* -*- Mode: C; c-file-style: "gnu" -*-
   deflater.c -- native methods for java/util/zip/Deflater
   Created: Chris Toshok <toshok@hungry.com>, 9-Nov-1997
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#include "config.h"
#include "jni.h"
#include "zlib.h"
#include "objects.h"

#include <assert.h>

static z_streamp
get_deflater_strm(JNIEnv *env,
		  jobject obj)
{
  jfieldID strm_field;
  jint strm;
  jclass obj_class = (*env)->GetObjectClass(env, obj);

  strm_field = (*env)->GetFieldID(env, obj_class, "strm", "I");
  strm = (*env)->GetIntField(env, obj, strm_field);

  return (z_streamp)strm;
}

static void
set_deflater_strm(JNIEnv *env,
		  jobject obj,
		  z_streamp strm)
{
  jfieldID strm_field;
  jclass obj_class = (*env)->GetObjectClass(env, obj);

  /* XXX This should be rewritten to use a lookup table */
  assert(sizeof(z_streamp) <= sizeof(jint));

  strm_field = (*env)->GetFieldID(env, obj_class, "strm", "I");
  (*env)->SetIntField(env, obj, strm_field, (jint)strm);
}

static jint
get_deflater_level(JNIEnv *env,
		   jobject obj)
{
  jfieldID level_field;
  jclass obj_class = (*env)->GetObjectClass(env, obj);

  level_field = (*env)->GetFieldID(env, obj_class, "level", "I");

  return (*env)->GetIntField(env, obj, level_field);
}

static jint
get_deflater_strategy(JNIEnv *env,
		      jobject obj)
{
  jfieldID strategy_field;
  jclass obj_class = (*env)->GetObjectClass(env, obj);

  strategy_field = (*env)->GetFieldID(env, obj_class, "strategy", "I");

  return (*env)->GetIntField(env, obj, strategy_field);
}

JNIEXPORT void JNICALL
Java_java_util_zip_Deflater_setDictionary(JNIEnv *env,
					  jobject obj,
					  jbyteArray b,
					  jint off,
					  jint len)
{
  z_streamp strm = get_deflater_strm(env, obj);
  jbyte* dict;

  dict = (*env)->GetByteArrayElements(env, b, NULL);

  deflateSetDictionary(strm,
		       dict + off,
		       len);

  (*env)->ReleaseByteArrayElements(env, b, dict, 0);
}

JNIEXPORT jint JNICALL
Java_java_util_zip_Deflater_deflate(JNIEnv *env,
				    jobject obj,
				    jbyteArray b,
				    jint off,
				    jint len)
{
  (*env)->FatalError(env, "java.util.zip.Deflater.deflate() not implemented");
  return 0;
}

JNIEXPORT jint JNICALL
Java_java_util_zip_Deflater_getAdler(JNIEnv *env,
				     jobject obj)
{
  z_streamp strm = get_deflater_strm(env, obj);

  return strm->adler;
}

JNIEXPORT jint JNICALL
Java_java_util_zip_Deflater_getTotalOut(JNIEnv *env,
					jobject obj)
{
  z_streamp strm = get_deflater_strm(env, obj);

  return strm->total_out;
}

JNIEXPORT jint JNICALL
Java_java_util_zip_Deflater_getTotalIn(JNIEnv *env,
				       jobject obj)
{
  z_streamp strm = get_deflater_strm(env, obj);

  return strm->total_in;
}

JNIEXPORT void JNICALL
Java_java_util_zip_Deflater_reset(JNIEnv *env,
				  jobject obj)
{
  z_streamp strm = get_deflater_strm(env, obj);

  deflateReset(strm);
}

JNIEXPORT void JNICALL
Java_java_util_zip_Deflater_end(JNIEnv *env,
				jobject obj)
{
  z_streamp strm = get_deflater_strm(env, obj);

  deflateEnd(strm);
}

JNIEXPORT void JNICALL
Java_java_util_zip_Deflater_init(JNIEnv *env,
				 jobject obj,
				 jboolean nowrap)
{
  z_streamp strm = get_deflater_strm(env, obj);
  jint level = get_deflater_level(env, obj);
  jint strategy = get_deflater_strategy(env, obj);

  deflateInit(strm, level);

  deflateParams(strm,
		level,
		strategy);
}
