/* -*- Mode: C; c-file-style: "gnu" -*-
   component.c -- native methods for sun/awt/motif/MComponentPeer
   Created: Chris Toshok <toshok@hungry.com>, 3-Mar-1998.
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */

#include "jni.h"

#include "common.h"

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pInitialize(JNIEnv *env,
					      jobject obj)
{
  printf ("In sun_awt_motif_MComponentPeer_pInitialize(peer = %p)\n", obj);
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pShow(JNIEnv *env,
					jobject peer)
{
  Widget w = get_component_widget(env, peer);

  printf ("in sun_awt_motif_MComponentPeer_pShow(peer = %p)\n", peer);

  LOCK_AWT();
  XtManageChild(w);
  UNLOCK_AWT();
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pHide(JNIEnv *env,
					jobject peer)
{
  Widget w = get_component_widget(env, peer);

  printf ("in sun_awt_motif_MComponentPeer_pHide(peer = %p)\n", peer);

  LOCK_AWT();
  XtUnmanageChild(w);
  UNLOCK_AWT();
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pEnable(JNIEnv *env,
					  jobject peer)
{
  Widget w = get_component_widget(env, peer);

  LOCK_AWT();
  XtSetSensitive(w, True);
  UNLOCK_AWT();
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pDisable(JNIEnv *env,
					   jobject peer)
{
  Widget w = get_component_widget(env, peer);

  LOCK_AWT();
  XtSetSensitive(w, False);
  UNLOCK_AWT();
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pReshape(JNIEnv *env,
					   jobject peer,
					   jint x,
					   jint y,
					   jint width,
					   jint height)
{
  Widget widget = get_component_widget(env, peer);

  LOCK_AWT();
  XtVaSetValues(widget,
		XmNx, x,
		XmNy, y,
		XmNwidth, width,
		XmNheight, height,
		NULL);
  UNLOCK_AWT();
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pAddRepaint(JNIEnv *env,
					      jobject obj,
					      jint x,
					      jint y,
					      jint width,
					      jint height)
{
  printf ("In sun_awt_motif_MComponentPeer_pAddRepaint\n");
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pTriggerRepaint(JNIEnv *env,
						  jobject obj)
{
  printf ("In sun_awt_motif_MComponentPeer_pTriggerRepaint\n");
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pDispose(JNIEnv *env,
					   jobject peer)
{
  Widget w = get_component_widget(env, peer);

  LOCK_AWT();
  XtDestroyWidget(w);
  UNLOCK_AWT();
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_pMakeCursorVisible(JNIEnv *env,
						     jobject obj)
{
  printf ("In sun_awt_motif_MComponentPeer_pMakeCursorVisible\n");
}

JNIEXPORT jobject JNICALL
Java_sun_awt_motif_MComponentPeer_pGetLocationOnScreen(JNIEnv *env,
						       jobject obj)
{
  printf ("In sun_awt_motif_MComponentPeer_pGetLocationOnScreen(peer = %p)\n", obj);
  return NULL;
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_setForeground(JNIEnv *env,
						jobject obj,
						jobject foreground)
{
  printf ("In sun_awt_motif_MComponentPeer_setForeground(peer = %p, foreground = %p)\n", obj, foreground);
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_setBackground(JNIEnv *env,
						jobject obj,
						jobject background)
{
  printf ("In sun_awt_motif_MComponentPeer_setBackground(peer = %p, background = %p)\n", obj, background);
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_setFont(JNIEnv *env,
					  jobject obj,
					  jobject font)
{
  printf ("In sun_awt_motif_MComponentPeer_setFont\n");
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_setCursor(JNIEnv *env,
					    jobject obj,
					    jobject cursor)
{
  printf ("In sun_awt_motif_MComponentPeer_setCursor\n");
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_handleEvent(JNIEnv *env,
					      jobject obj,
					      jobject event)
{
  printf ("In sun_awt_motif_MComponentPeer_handleEvent\n");
}

JNIEXPORT void JNICALL
Java_sun_awt_motif_MComponentPeer_requestFocus(JNIEnv *env,
					       jobject obj)
{
  printf ("In sun_awt_motif_MComponentPeer_requestFocus\n");
}
