/* -*- Mode: C; c-file-style: "gnu" -*-
   abort.c -- our own version of abort that calls the vm hook.
   Created: Chris Toshok <toshok@hungry.com>, 15-Aug-1997
*/
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#include "config.h"
#include "jniint.h"
#include "native-threads.h"
#include "interp.h"

#include <signal.h>
#include <unistd.h>

void
abort()
{
  JNIEnv *env = THREAD_getEnv();
  if (env)
    {
      HungryJNIEnv *henv = (HungryJNIEnv*)env;
      HungryJavaVM *vm = henv->_vm;

      if (vm->_args.abort)
	(*vm->_args.abort)();
    }

  raise(SIGBUS);
  /* NOTREACHED */
}

void
fatal_signal_handler()
{
  JThreadInfo* thread_info = THREAD_getJavaInfo();
  StackFrame *frame = TOPFRAME(thread_info);
  struct sigaction sigact;

  sigact.sa_handler = SIG_DFL;
  sigemptyset(&sigact.sa_mask);
  sigact.sa_flags = 0;

  sigaction(SIGBUS, &sigact, NULL);
  sigaction(SIGSEGV, &sigact, NULL);

  while (frame < thread_info->stack_highwater)
    {
      if (frame->flags & FRAME_NATIVE)
	{
	  fprintf (stderr, "native frame:\t%s.%s\n",
		   CLASSNAME(frame),
		   METHODNAME(frame));
	}
      else 
	{
	  fprintf (stderr, "java frame:\t%s.%s (pc = %d)\n",
		   CLASSNAME(frame),
		   METHODNAME(frame), PC(frame));
	}

      frame = get_frame_parent(frame);
    }

  raise(SIGBUS);
}
