/*
 * This file contains the code necessary to fake out the rest
 * of the runtime into thinking that we actually do have array
 * classes.
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#include "config.h"
#include "jni.h"
#include "ClazzFile.h"
#include "log.h"
#include "resolve.h"
#include "arrays.h"
#include "array-class.h"
#include "class-repository.h"

#include <string.h>
#include <stdlib.h>

#define MYLOG "ArrayClass"

static ClassConstant superclass_constant = {
  CONSTANT_Class,
  2
};

static Utf8Constant object_name_constant = {
  CONSTANT_Utf8,
  16, /* strlen("java/lang/Object") */
  NULL
};

jclass
createFakeArrayClass(JNIEnv *env, char *class_name)
{
  ClazzFile *class = find_class_in_repository(env, class_name);

  /* we've already created it. */
  if (class)
    return class;

  JAVARLOG1(MYLOG, 1, "creating array class %s\n",
	    class_name);

  class = (ClazzFile*)calloc(1, sizeof(ClazzFile));

  class->access_flags = ACC_ARRAY | ACC_PUBLIC;
  class->constant_pool_count = 4;
  class->superclass_index = 1;
  class->superclass = (ClazzFile*)find_class(env, "java/lang/Object");
  class->class_name = strdup(class_name);
  class->initialization_state = CLASS_INIT_FINISHED;
  class->class_name = strdup(class_name);

  class->constants = (ConstantPoolEntry*)calloc(3, sizeof(ConstantPoolEntry));
  class->constants[1].clazz_info = superclass_constant;
  class->constants[2].utf8_info = object_name_constant;
  class->constants[2].utf8_info.bytes = (u1*)strdup("java/lang/Object");

  class->num_fields = 4;
  class->num_instance_fields = 4;

  class->fields = (FieldStruct**)calloc(class->num_fields, sizeof(FieldStruct*));

  class->fields[ARRAY_LENGTH_INDEX] = (FieldStruct*)calloc(1, sizeof(FieldStruct));
  class->fields[ARRAY_LENGTH_INDEX]->clazz = class;
  class->fields[ARRAY_LENGTH_INDEX]->name = "length";
  class->fields[ARRAY_LENGTH_INDEX]->sig = SIG_parseFromJavaSig(env, "I");
  class->fields[ARRAY_LENGTH_INDEX]->access_flags = ACC_PUBLIC;
  class->fields[ARRAY_LENGTH_INDEX]->java_type = SIG_JINT;
  class->fields[ARRAY_LENGTH_INDEX]->field_index = ARRAY_LENGTH_INDEX;

  class->fields[ARRAY_TYPE_INDEX] = (FieldStruct*)calloc(1, sizeof(FieldStruct));
  class->fields[ARRAY_TYPE_INDEX]->clazz = class;
  class->fields[ARRAY_TYPE_INDEX]->name = "type";
  class->fields[ARRAY_TYPE_INDEX]->sig = SIG_parseFromJavaSig(env, "I");
  class->fields[ARRAY_TYPE_INDEX]->access_flags = ACC_PUBLIC;
  class->fields[ARRAY_TYPE_INDEX]->java_type = SIG_JINT;
  class->fields[ARRAY_TYPE_INDEX]->field_index = ARRAY_TYPE_INDEX;

  class->fields[ARRAY_BODY_INDEX] = (FieldStruct*)calloc(1, sizeof(FieldStruct));
  class->fields[ARRAY_BODY_INDEX]->clazz = class;
  class->fields[ARRAY_BODY_INDEX]->name = "body";
  class->fields[ARRAY_BODY_INDEX]->sig = NULL;
  class->fields[ARRAY_BODY_INDEX]->access_flags = ACC_PUBLIC;
  class->fields[ARRAY_BODY_INDEX]->java_type = SIG_JINT; /** XXX can't make it SIG_JOBJECT or the garbage collector
							   will try to traverse to it... badness. */
  class->fields[ARRAY_BODY_INDEX]->field_index = ARRAY_BODY_INDEX;

  class->fields[ARRAY_ELSIZE_INDEX] = (FieldStruct*)calloc(1, sizeof(FieldStruct));
  class->fields[ARRAY_ELSIZE_INDEX]->clazz = class;
  class->fields[ARRAY_ELSIZE_INDEX]->name = "elsize";
  class->fields[ARRAY_ELSIZE_INDEX]->sig = SIG_parseFromJavaSig(env, "I");
  class->fields[ARRAY_ELSIZE_INDEX]->access_flags = ACC_PUBLIC;
  class->fields[ARRAY_ELSIZE_INDEX]->java_type = SIG_JINT;
  class->fields[ARRAY_ELSIZE_INDEX]->field_index = ARRAY_ELSIZE_INDEX;

  JAVARLOG0(MYLOG, 1, "  adding array class to repository\n");

  add_class_to_repository(env, class, class_name);

  return (jclass)class;
}

jclass
createFakeArrayRemoveDimension(JNIEnv *env, jclass array)
{
  char *arrayclass_name = getClassName(env, (ClazzFile*)array);

  return createFakeArrayClass(env, arrayclass_name + 1);
}
