/* -*- Mode: C; c-file-style: "gnu" -*-
   printf.c -- our own version of printf/fprintf that calls the vm hook.
   Created: Chris Toshok <toshok@hungry.com>, 15-Aug-1997
*/
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#include "config.h"
#include "jniint.h"
#include "native-threads.h"

#include <stdarg.h>
#include <stdio.h>

int
printf (const char *format,
	...)
{
  JNIEnv *env = THREAD_getEnv();
  va_list args;
  int result;

  va_start(args, format);

  if (env)
    {
      HungryJNIEnv *henv = (HungryJNIEnv*)env;
      if (henv->_vm
	  && henv->_vm->_args.vfprintf)
	result = (*henv->_vm->_args.vfprintf)(stdout, format, args);
      else
	result = vfprintf(stdout, format, args);
    }
  else
    result = vfprintf(stdout, format, args);

  va_end(args);

  return result;
}

int
fprintf (FILE *fp,
	 const char *format,
	 ...)
{
  JNIEnv *env = THREAD_getEnv();
  va_list args;
  int result;

  va_start(args, format);

  if (env)
    {
      HungryJNIEnv *henv = (HungryJNIEnv*)env;
      if (henv->_vm
	  && henv->_vm->_args.vfprintf)
	result = (*henv->_vm->_args.vfprintf)(fp, format, args);
      else
	result = vfprintf(fp, format, args);
    }
  else
    result = vfprintf(fp, format, args);

  va_end(args);

  return result;
}
