/* -*- Mode: C; c-file-style: "gnu" -*-
   dump-class.c -- print out a class file.
   Created: Chris Toshok <toshok@hungry.com>, 5-Dec-1997.
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "jni.h"
#include "resolve.h"
#include "dump-class.h"

#include <string.h>
#include <stdlib.h>
#include <unistd.h>

jboolean as_html = JNI_FALSE;

JNIEnv *the_env;

static void
usage()
{
  fprintf(stderr, "dump-class usage:  dump-class [-txt|-html] ClassName1 ClassName2 ...\n");
  fprintf(stderr, "   -txt  :  Output the files as plain ascii text. (default)\n");
  fprintf(stderr, "   -html :  Output the files using HTML.\n");
  exit(0);
}

int
main(int argc,
     char **argv)
{
  JavaVM *jvm;
  JDK1_1InitArgs vm_args;
  int i;
  int initial_class_pos = 1;

  jclass cls;

  if (argc == 1)
    {
      usage();
    }
  else
    {
      if (!strcmp(argv[1], "-html")
	  || !strcmp(argv[1], "--html"))
	{
	  as_html = JNI_TRUE;
	  initial_class_pos = 2;
	}
      else if (!strcmp(argv[1], "-txt")
	       || !strcmp(argv[1], "--txt"))
	{
	  as_html = JNI_FALSE;
	  initial_class_pos = 2;
	}
    }

  /*
  ** Get the default initialization arguments and set the class path.
  */
  vm_args.version = 0x10001; /* JDK 1.1 */
  JNI_GetDefaultJavaVMInitArgs(&vm_args);

  /*
  ** load and initialize a JavaVM, return a JNI interface pointer in
  ** env.
  */
  JNI_CreateJavaVM(&jvm, &the_env, &vm_args);

  /* now for each other argument, disassemble the class */
  for (i = initial_class_pos; i < argc; i ++)
    {
      cls = (*the_env)->FindClass(the_env, argv[i]);

      if (cls == NULL)
	{
	  printf("Unable to load class %s\n", argv[i]);
	  (*the_env)->ExceptionClear(the_env);
	}
      else
	{
	  FILE *fp;
	  char *filename;

	  if (as_html)
	    {
	      filename = (char*)malloc(strlen(argv[i]) + strlen(".html") + 1);
	      sprintf (filename, "%s.html", argv[i]);
	    }
	  else
	    {
	      filename = (char*)malloc(strlen(argv[i]) + strlen(".out") + 1);
	      sprintf (filename, "%s.out", argv[i]);
	    }

	  fp = fopen(filename, "w");
	  if (fp == NULL)
	    {
	      printf ("Couldn't open file '%s' for writing.\n", filename);
	      free(filename);
	      continue;
	    }

	  output_header(fp, argv[i]);
	  spew_clazz(fp, (ClazzFile*)cls);
	  output_footer(fp);

	  free(filename);

	  fclose(fp);
	}
    }

  /* keep 'gcc -Wall -pedantic' happy */
  exit(0);
}

