/* -*- Mode: C; c-file-style: "gnu" -*-
   japharh.c -- output headers and stubs for java class files.
   Created: Chris Toshok <toshok@hungry.com>
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>

#include "classpath.h"
#include "resolve.h"
#include "log.h"
#include "japharh.h"

#define MYLOG "Javah"

char *javah_dest_dir = NULL;
int do_stubs = 0;

extern void output_header(ClazzFile*, char*);
extern void output_stubs(ClazzFile*, char*);

void usage()
{
  fprintf (stderr, "usage:  javah [-d directory] [-stubs] [-classpath classpath]+ classes...\n");
  exit(0);
}

int parse_command_line(int argc, char *argv[])
{
  int i;

  for (i = 1; i < argc; i ++)
    {
      if (argv[i][0] == '-')
	{
	  if (!strcmp(argv[i], "-classpath"))
	    {
	      /* do more here... */
	      i ++;
	    }
	  else if (!strcmp(argv[i], "-d"))
	    {
	      javah_dest_dir = strdup(argv[i + 1]);

	      i ++;
	    }
	  else if (!strcmp(argv[i], "-stubs"))
	    {
	      do_stubs = 1;
	    }
	  else 
	    {
	      fprintf (stderr, "%s:  Illegal command line argument '%s'\n", argv[0], argv[i]);
	      usage();
	    }
	}
      else
	return i;
    }

  return -1;
}

void
check_dest_dir(char *dest_dir)
{
  struct stat buf;
  int stat_result;

  stat_result = stat(dest_dir, &buf);

  if (stat_result != -1
      && !S_ISDIR(buf.st_mode))
    {
      fprintf (stderr, "Destination directory %s is not a directory\n", dest_dir);
      exit(1);
    }
  else if (stat_result == -1)
    {
      if (mkdir(dest_dir, 0777) == -1)
	{
	  fprintf (stderr, "Could not create the destination directory %s\n", dest_dir);
	  exit(1);
	}
    }
}

int main (int argc, char *argv[])
{
  ClazzFile *cf;
  ClasspathEntry *entries;
  int num_cp_entries;
  int first_class_index;
  int i;
  JNIEnv *env = NULL;
  char *CLASSPATH;

  first_class_index = parse_command_line(argc, argv);

  CLASSPATH = getenv("CLASSPATH");

  if (CLASSPATH == NULL)
    CLASSPATH=CLASSPATH_getSystemClasspath();

  CLASSPATH_create(CLASSPATH, &entries, &num_cp_entries);

  if (javah_dest_dir == NULL)
    javah_dest_dir = "./";

  if (first_class_index == -1)
    {
      usage();
    }

  check_dest_dir(javah_dest_dir);

  for (i = first_class_index; i < argc; i ++)
    {
      char *class_name = argv[i];
      char *slash_class_name = strdup(class_name);

      dots_to_slashes(slash_class_name);

      JAVARLOG1(MYLOG, 1, "  Processing class '%s'...\n", class_name);

      cf = find_class_on_classpath(env, slash_class_name, entries, num_cp_entries);
      
      if (cf == NULL || NULL != strchr(class_name, '/'))
	{
	  fprintf (stderr, "Unable to load class %s\n", class_name);
	  exit(1);
	}
      
      dots_to_underscores(class_name);
      
      if (do_stubs)
        output_stubs(cf, class_name);
      else
        output_header(cf, class_name);

      free(slash_class_name);
    }

  return 0;
}
