/* -*- Mode: C; c-file-style: "gnu" -*-
   stubs.c -- special code for outputting stubs for japharh.
   Created: Chris Toshok <toshok@hungry.com>
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include <string.h>
#include <ClazzFile.h>
#include <assert.h>
#include <japharh.h>
#include <log.h>
#include <sig.h>

#define MYLOG "Javah"

extern char *javah_dest_dir;

void
output_method_stub(FILE *fp,
		   char *class_name,
		   char *method_name,
		   Signature *sig,
		   int isStatic)
{
  int initial_param;
  JNIEnv *env = NULL;

  JAVARLOG0(MYLOG, 1, " in output_method_stub()\n");

  JAVARLOG3(MYLOG, 2, "   class_name = '%s' method_name = '%s' method_sig = '%s'\n",
	    class_name, method_name, SIG_formatToC(env, sig));

  fprintf(fp, "/* \"%s/%s()\",\n   Java_%s_%s_stub  */\n",
	  class_name, method_name,
	  class_name, method_name);

  fprintf(fp, "jvalue *\n");
  fprintf(fp, "Java_%s_%s_stub ( JNIEnv *env, jvalue *p )\n", class_name, method_name);
  fprintf(fp, "{\n");

  /* now we write out the prorotype for the non-stub function. */
  fprintf(fp, "   extern %s Java_%s_%s( JNIEnv *env, %s",
	  SIG_formatToC(env, sig->method.return_type),
	  class_name, method_name,
	  isStatic ? "jclass cls" : "jobject obj");

  if (SIG_numParams(env, sig) > 0)
    {
      int i;
	  
      fprintf (fp, ", ");      
      for (i = 0; i < SIG_numParams(env, sig); i ++)
	{
	  fprintf (fp, "%s%s", 
		   ( sig->method.params[i]->any.tag == SIG_CLASS ? "jobject"
		     : SIG_formatToC(env, sig->method.params[i])) ,
		   i < SIG_numParams(env, sig) - 1 ? ", " : "");
	}
    }
  
  fprintf (fp, ");\n");

  /* now we actually output the call to the function. 
     the return value (if there is one) is stored in p[0] */

  fprintf (fp, "   ");

  if (!SIG_isVoid(env, sig->method.return_type))
    fprintf (fp, "p[0].%c = ", 
	     SIG_toUnionSelector(env, sig->method.return_type));

  fprintf (fp, "Java_%s_%s( env, p[1].l", class_name, method_name);

  initial_param = 2;

  if (SIG_numParams(env, sig) > 0)
    {
      int i;
      
      fprintf(fp, ", ");
	  
      for (i = 0; i < SIG_numParams(env, sig); i ++)
	{
	  fprintf (fp, "p[%d].%c%s",
		   i + initial_param , 
		   SIG_toUnionSelector(env, sig->method.params[i]),
		   i < SIG_numParams(env, sig) - 1 ? ", " : "");
	}
    }
  fprintf (fp, ");\n\n");

  fprintf (fp, "  return p;\n");

  fprintf(fp, "}\n\n");
}

void
output_stubs(ClazzFile *cf, char *bar_name)
{
  char full_name[100];
  FILE *fp;
  int i;

  strcpy(full_name, javah_dest_dir);
  if (javah_dest_dir[strlen(javah_dest_dir) - 1] != '/')
    strcat(full_name, "/");
  strcat(full_name, bar_name);
  strcat(full_name, "_stubs.c");

  fp = fopen(full_name, "w");
  if (NULL == fp) {
    fprintf(stderr, "Unable to open %s for writing.\n", full_name);
    return;
  }

  fprintf (fp, "/* DO NOT EDIT THIS FILE - it is machine generated by japharh */\n");
  fprintf (fp, "#include \"jni.h\"\n\n");

  fprintf (fp, "/* Stubs for class '%s' */\n\n", bar_name);

  for (i = 0; i < cf->num_methods; i ++)
    if (cf->methods[i]->access_flags & ACC_NATIVE)
      output_method_stub(fp,
			 bar_name,
			 cf->methods[i]->name,
			 cf->methods[i]->sig,
			 (cf->methods[i]->access_flags & ACC_STATIC));

  fclose(fp);
}

