import java.net.*;

/*
 * Start one thread for receiving a packet, wait for it to set up,
 * send a packet to it, and wait until it completes.  Compare the
 * packet to make sure it came thru without errors.
 */

public class DatagramSocketSendReceiveTest
  implements Runnable
{
  public static final int port = 4000 + (int)(java.lang.Math.random() * 1000);
  public static final byte[] message = {'h','e','l','l','o'};
  public static int count = 0;

  void send()
    throws Exception
    {
      DatagramSocket sender = new DatagramSocket();
      InetAddress local = sender.getLocalAddress();

      DatagramPacket packet = new DatagramPacket(message, message.length,
						 local, port);

      sender.send(packet);
      sender.close();
    }
  void receive()
    throws Exception
    {
      DatagramSocket socket = new DatagramSocket(port);
      socket.setSoTimeout(10);

      byte[] buffer = new byte[100];
      DatagramPacket packet = new DatagramPacket(buffer, buffer.length);

      synchronized(this) {
	notifyAll();
      }

      socket.receive(packet);

      socket.close();

      count++;
      if ( message.length != packet.getLength() )
	throw new Exception("Receved "+ packet.getLength()+
			    " but sent "+message.length);

    }

  public void run()
    {
      try {
	receive();
      } catch (Exception e) {
	System.out.println("FAILED: receiver: " + e);
	System.exit(1);
      }
    }
  public static void main(String args[])
    {
      try {
	DatagramSocketSendReceiveTest sender =
	  new DatagramSocketSendReceiveTest();
	DatagramSocketSendReceiveTest receiver =
	  new DatagramSocketSendReceiveTest();
	Thread receiver_thread = new Thread(receiver);

	synchronized(receiver) {
	  receiver_thread.start();
	  receiver.wait();
	}
	try {
	  sender.send();
	} catch (Exception e) {
	  System.out.println("FAILED: sender: " + e);
	  System.exit(1);
	}
	receiver_thread.join();
	  
	if (0 == count)
	  throw new Exception("Nothing received");

	System.out.println("PASSED: DatagramSocket send/receive count="+count);
      } catch (Exception e) {
	System.out.println("FAILED: " + e);
	System.exit(1);
      }
    }
}
