import java.net.*;
import java.io.*;

/*
 * Start one thread for receiving a packet, wait for it to set up,
 * send a packet to it, and wait until it completes.  Compare the
 * packet to make sure it came thru without errors.
 */

public class SocketSendReceiveTest
  implements Runnable
{
  public static final int port = 4000 + (int)(java.lang.Math.random() * 2000);
  public static final byte[] message = {'h','e','l','l','o'};
  public static int count = 0;

  void send()
    throws Exception
    {
      InetAddress local = InetAddress.getLocalHost();
      Socket sender = new Socket(local, port);

      DataOutputStream out = new DataOutputStream(sender.getOutputStream());
      out.write(message, 0, message.length);
      out.flush();
      sender.close();
    }
  void receive()
    throws Exception
    {
      ServerSocket socket = new ServerSocket(port);

      synchronized(this) {
	  notifyAll();
      }

      Socket connection = socket.accept();
      DataInputStream in = new DataInputStream(connection.getInputStream());

      byte[] buffer = new byte[100];

      int length = in.read(buffer);

      connection.close();
      socket.close();

      count++;
      if ( message.length != length )
	throw new Exception("Receved "+ length + " but sent "+message.length);
    }

  public void run()
    {
      String name = Thread.currentThread().getName();
      try {
	receive();
      } catch (Exception e) {
	System.out.println("FAILED: receiver (port "+port + "): " + e);
	System.exit(1);
      }
    }
  public static void main(String args[])
    {
      try {
	SocketSendReceiveTest sender = new SocketSendReceiveTest();
	SocketSendReceiveTest receiver = new SocketSendReceiveTest();
	Thread receiver_thread = new Thread(receiver);

	synchronized(receiver) {
	  receiver_thread.start();
	  receiver.wait();
	}
	try {
	  sender.send();
	} catch (Exception e) {
	  System.out.println("FAILED: receiver (port "+port + "): " + e);
	  System.exit(1);
	}
	receiver_thread.join();

	if (0 == count)
	  throw new Exception("Nothing received");

	System.out.println("PASSED: DatagramSocket send/receive count="+count);
      } catch (Exception e) {
	System.out.println("FAILED: " + e);
	System.exit(1);
      }
    }
}
